"""Main metron_tagger tests"""
import io
import sys
from pathlib import Path

import pytest
from darkseid.comicarchive import ComicArchive
from mokkari.issues_list import IssuesList

from metrontagger.main import (
    SETTINGS,
    delete_comics_metadata,
    list_comics_with_missing_metadata,
    sort_list_of_comics,
)
from metrontagger.talker import MultipleMatch, Talker


def test_create_metron_talker():
    SETTINGS.metron_user = "test"
    SETTINGS.metron_pass = "test_password"
    talker = Talker(SETTINGS.metron_user, SETTINGS.metron_pass)
    assert isinstance(talker, Talker)


def test_list_comics_with_missing_metadata(fake_comic):
    expected_result = (
        "\nShowing files without metadata:"
        + "\n-------------------------------"
        + "\nno metadata in 'Aquaman v1 #001 (of 08) (1994).cbz'"
        + "\n"
    )
    # Make sure fake comic archive doesn't have any metadata
    if ComicArchive(fake_comic).has_metadata():
        ComicArchive(fake_comic).remove_metadata()

    fake_list = [fake_comic]

    # Capture the output so we can verify the print output
    captured_output = io.StringIO()
    sys.stdout = captured_output

    list_comics_with_missing_metadata(fake_list)
    sys.stdout = sys.__stdout__

    assert expected_result == captured_output.getvalue()


def test_delete_comics_with_metadata(fake_comic, fake_metadata):
    expected_result = (
        "\nRemoving metadata:\n-----------------"
        + "\nremoved metadata from 'Aquaman v1 #001 (of 08) (1994).cbz'"
        + "\n"
    )

    # If comic doesn't already have metadata let's add it
    if not ComicArchive(fake_comic).has_metadata():
        ComicArchive(fake_comic).write_metadata(fake_metadata)

    fake_list = [fake_comic]

    # Capture the output so we can verify the print output
    captured_output = io.StringIO()
    sys.stdout = captured_output

    delete_comics_metadata(fake_list)
    sys.stdout = sys.__stdout__

    assert expected_result == captured_output.getvalue()


def test_delete_comics_without_metadata(fake_comic, fake_metadata):
    expected_result = (
        "\nRemoving metadata:\n-----------------"
        + "\nno metadata in 'Aquaman v1 #001 (of 08) (1994).cbz'"
        + "\n"
    )

    # If comic has metadata let's remove it
    if ComicArchive(fake_comic).has_metadata():
        ComicArchive(fake_comic).remove_metadata()

    fake_list = [fake_comic]

    # Capture the output so we can verify the print output
    captured_output = io.StringIO()
    sys.stdout = captured_output

    delete_comics_metadata(fake_list)
    sys.stdout = sys.__stdout__

    assert expected_result == captured_output.getvalue()


def test_sort_comics_without_sort_dir(fake_comic, tmpdir):
    expected_result = "\nUnable to sort files. No destination directory was provided.\n"

    # Create fake settings.
    SETTINGS.sort_dir = ""

    fake_list = [fake_comic]

    # Capture the output so we can verify the print output
    captured_output = io.StringIO()
    sys.stdout = captured_output

    sort_list_of_comics(fake_list)
    sys.stdout = sys.__stdout__

    assert expected_result == captured_output.getvalue()


def test_sort_comics_with_dir(fake_comic, fake_metadata, tmpdir):
    SETTINGS.sort_dir = tmpdir

    expected_result = (
        "\nStarting sorting of comic archives:\n----------------------------------\n"
        + "moved 'Aquaman v1 #001 (of 08) (1994).cbz' to "
        + f"'{SETTINGS.sort_dir}/{fake_metadata.publisher}/{fake_metadata.series}/v{fake_metadata.volume}'\n"
    )

    comic = ComicArchive(fake_comic)

    if not comic.has_metadata():
        comic.write_metadata(fake_metadata)

    fake_list = [fake_comic]

    # Capture the output so we can verify the print output
    captured_output = io.StringIO()
    sys.stdout = captured_output

    sort_list_of_comics(fake_list)
    sys.stdout = sys.__stdout__

    # Path for moved file
    moved_comic = (
        Path(f"{SETTINGS.sort_dir}")
        / f"{fake_metadata.publisher}"
        / f"{fake_metadata.series}"
        / f"v{fake_metadata.volume}"
        / fake_comic.name
    )

    assert moved_comic.parent.is_dir()
    assert moved_comic.is_file()
    assert expected_result == captured_output.getvalue()


@pytest.fixture()
def multi_choice_fixture():
    i = {
        "count": 2,
        "next": None,
        "previous": None,
        "results": [
            {"issue_name": "Superman #1", "cover_date": "1939-10-01"},
            {"issue_name": "Superman #1", "cover_date": "1986-01-01"},
        ],
    }
    return IssuesList(i)


def test_print_multi_choices_to_user(talker, multi_choice_fixture, capsys):
    fn = Path("/tmp/Superman #1")
    test_data = MultipleMatch(fn, multi_choice_fixture)
    expected_result = "1. Superman #1 (1939-10-01)\n2. Superman #1 (1986-01-01)\n"
    talker._print_choices_to_user(test_data.matches)
    stdout, _ = capsys.readouterr()
    assert stdout == expected_result


def test_post_process_matches(capsys, talker):
    talker.match_results.add_good_match("Inhumans #1.cbz")
    talker.match_results.good_matches.append("Inhumans #2.cbz")
    talker.match_results.no_matches.append("Outsiders #1.cbz")
    talker.match_results.no_matches.append("Outsiders #2.cbz")

    expected_result = (
        "\nSuccessful matches:\n------------------\nInhumans #1.cbz\nInhumans #2.cbz\n\n"
    )
    expected_result += "No matches:\n------------------\nOutsiders #1.cbz\nOutsiders #2.cbz\n"

    talker._post_process_matches()
    stdout, _ = capsys.readouterr()

    assert stdout == expected_result



import zipfile

import pytest
from darkseid.genericmetadata import GenericMetadata

from metrontagger.options import make_parser
from metrontagger.talker import Talker

CONTENT = "blah blah blah"


@pytest.fixture(scope="session")
def parser():
    return make_parser()


@pytest.fixture(scope="session")
def talker():
    username = "Foo"
    password = "Bar"
    return Talker(username, password)


@pytest.fixture(scope="function")
def fake_metadata():
    meta_data = GenericMetadata()
    meta_data.publisher = "DC Comics"
    meta_data.series = "Aquaman"
    meta_data.volume = "2"
    meta_data.issue = "1"
    meta_data.year = "2011"
    meta_data.add_credit("Peter David", "Writer")
    meta_data.add_credit("Martin Egeland", "Penciller")
    meta_data.add_credit("Martin Egeland", "Cover")

    return meta_data


@pytest.fixture(scope="function")
def fake_comic(tmp_path_factory):
    test_dir = tmp_path_factory.mktemp("data")
    img_1 = test_dir / "image-1.jpg"
    img_1.write_text(CONTENT)
    img_2 = test_dir / "image-2.jpg"
    img_2.write_text(CONTENT)
    img_3 = test_dir / "image-3.jpg"
    img_3.write_text(CONTENT)

    z_file = tmp_path_factory.mktemp("comic") / "Aquaman v1 #001 (of 08) (1994).cbz"
    zf = zipfile.ZipFile(z_file, "w")
    try:
        zf.write(img_1)
        zf.write(img_2)
        zf.write(img_3)
    finally:
        zf.close()

    return z_file



from metrontagger.utils import cleanup_string, create_query_params


def test_dict(tmp_path):
    series = "Aquaman"
    number = "9"
    volume = "1"
    year = "1999"
    # Make the tmp file
    comic = tmp_path / f"{series} v{volume} #{number} ({year}).cbz"

    result = create_query_params(comic)
    expected = {
        "series_name": f"{series}",
        "series_volume": f"{volume}",
        "number": f"{number}",
        "cover_year": f"{year}",
    }
    assert result == expected


def test_query_dict_without_issue_number(tmp_path):
    series = "Batman"
    year = "1990"
    volume = "2"
    # Make the tmp file
    comic = tmp_path / f"{series} v{volume} ({year}).cbz"

    result = create_query_params(comic)
    expected = {
        "series_name": f"{series}",
        "series_volume": f"{volume}",
        "number": "1",
        "cover_year": f"{year}",
    }
    assert result == expected


def test_cleanup_colon_space():
    test_str = "Hashtag: Danger (2019)"
    res = cleanup_string(test_str)
    assert res == "Hashtag - Danger (2019)"


def test_cleanup_space_colon():
    test_str = "Hashtag :Danger (2019)"
    res = cleanup_string(test_str)
    assert res == "Hashtag -Danger (2019)"


def test_cleanup_colon():
    test_str = "Hashtag:Danger (2019)"
    res = cleanup_string(test_str)
    assert res == "Hashtag-Danger (2019)"


def test_cleanup_backslash():
    test_str = "Hack/Slash (2019)"
    res = cleanup_string(test_str)
    assert res == "Hack-Slash (2019)"


def test_cleanup_questionmark():
    test_str = "What If? (2019)"
    res = cleanup_string(test_str)
    assert res == "What If (2019)"



def test_creditial_options(parser, tmpdir):
    parsed = parser.parse_args(
        [
            "--user",
            "test_user",
            "--password",
            "test_passwd",
            "--set-metron-user",
            str(tmpdir),
        ]
    )

    assert parsed.user == "test_user"
    assert parsed.password == "test_passwd"
    assert parsed.set_metron_user is True
    assert parsed.path == [str(tmpdir)]


def test_online_option(parser, tmpdir):
    parsed = parser.parse_args(["-o", str(tmpdir)])
    assert parsed.online is True
    assert parsed.path == [str(tmpdir)]


def test_id_option(parser, tmpdir):
    parsed = parser.parse_args(["--id", "1", str(tmpdir)])
    assert parsed.id == "1"
    assert parsed.path == [str(tmpdir)]


def test_delete_option(parser, tmpdir):
    parsed = parser.parse_args(["-d", str(tmpdir)])
    assert parsed.delete is True
    assert parsed.path == [str(tmpdir)]


def test_missing_option(parser, tmpdir):
    parsed = parser.parse_args(["--missing", str(tmpdir)])
    assert parsed.missing is True
    assert parsed.path == [str(tmpdir)]


def test_rename_option(parser, tmpdir):
    parsed = parser.parse_args(["-r", str(tmpdir)])
    assert parsed.rename is True
    assert parsed.path == [str(tmpdir)]


def test_ignore_option(parser, tmpdir):
    parsed = parser.parse_args(["--ignore-existing", str(tmpdir)])
    assert parsed.ignore_existing is True
    assert parsed.path == [str(tmpdir)]



from darkseid.comicarchive import ComicArchive

from metrontagger.filesorter import FileSorter


def test_sort_comic_with_missing_metadata(fake_comic, fake_metadata, tmp_path):
    test_dir = tmp_path / "sort10"

    fake_metadata.volume = None

    comic = ComicArchive(fake_comic)
    if comic.has_metadata():
        comic.remove_metadata()
    comic.write_metadata(fake_metadata)
    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert res is False


def test_sort_comic(fake_comic, fake_metadata, tmp_path):
    test_dir = tmp_path / "sort1"

    test_dir.mkdir()

    result_dir = (
        test_dir / fake_metadata.publisher / fake_metadata.series / f"v{fake_metadata.volume}"
    )

    # Write metadata to fake file
    comic = ComicArchive(fake_comic)
    if comic.has_metadata():
        comic.remove_metadata()
    comic.write_metadata(fake_metadata)

    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert result_dir.is_dir()
    assert res is True


def test_sort_files_without_metadata(fake_comic, tmp_path):
    test_dir = tmp_path / "sort2"
    # If we add more tests we should probably create another tmpfile
    # since we are removing the metadata from the tmpfile
    comic = ComicArchive(fake_comic)
    comic.remove_metadata()

    file_sorter = FileSorter(test_dir)
    res = file_sorter.sort_comics(fake_comic)
    assert res is False



import pytest
from darkseid.comicarchive import ComicArchive
from darkseid.utils import list_to_string
from mokkari.issue import IssueSchema
from mokkari.issues_list import IssuesList
from mokkari.session import Session


@pytest.fixture()
def metron_response():
    i = {
        "id": 31047,
        "publisher": {"id": 1, "name": "Marvel"},
        "series": {"id": 2222, "name": "The Spectacular Spider-Man"},
        "volume": 1,
        "number": "47",
        "name": ["A Night on the Prowl!"],
        "cover_date": "1980-10-01",
        "store_date": None,
        "desc": "Spider-Man goes on a wild goose chase to find out who is behind the Prowler impersonation.",
        "image": "https://static.metron.cloud/media/issue/2021/05/22/the-spectacular-spider-man-47.jpg",
        "arcs": [{"id": 1, "name": "Foo Bar"}],
        "credits": [
            {"id": 233, "creator": "Al Milgrom", "role": [{"id": 7, "name": "Cover"}]},
        ],
        "characters": [
            {"id": 6784, "name": "Debra Whitman"},
            {"id": 3067, "name": "Hobgoblin (Kingsley)"},
        ],
        "teams": [{"id": 1, "name": "Bar Foo"}],
    }
    return IssueSchema().load(i)


def test_map_resp_to_metadata(talker, metron_response):
    md = talker._map_resp_to_metadata(metron_response)
    assert md is not None
    assert md.title == list_to_string([title for title in metron_response.story_titles])
    assert md.series == metron_response.series.name
    assert md.volume == metron_response.volume
    assert md.publisher == metron_response.publisher.name
    assert md.issue == metron_response.number
    assert md.story_arc == list_to_string([a.name for a in metron_response.arcs])
    assert md.teams == list_to_string([t.name for t in metron_response.teams])
    assert md.year == metron_response.cover_date.year
    assert md.characters == list_to_string([c.name for c in metron_response.characters])
    assert md.credits is not None
    assert md.credits[0]["person"] == "Al Milgrom"
    assert md.credits[0]["role"] == "Cover"


def test_map_resp_to_metadata_with_no_story_name(talker, metron_response):
    test_data = metron_response
    test_data.story_titles = None
    meta_data = talker._map_resp_to_metadata(test_data)
    assert meta_data is not None
    assert meta_data.title is None
    assert meta_data.series == metron_response.series.name
    assert meta_data.volume == metron_response.volume
    assert meta_data.publisher == metron_response.publisher.name
    assert meta_data.issue == metron_response.number
    assert meta_data.year == metron_response.cover_date.year


@pytest.fixture()
def issue_list_response():
    i = {
        "count": 8,
        "next": None,
        "previous": None,
        "results": [
            {"id": 3634, "__str__": "Aquaman #1", "cover_date": "1962-02-01"},
            {"id": 2471, "__str__": "Aquaman #1", "cover_date": "1986-02-01"},
            {"id": 2541, "__str__": "Aquaman #1", "cover_date": "1989-06-01"},
            {"id": 2510, "__str__": "Aquaman #1", "cover_date": "1991-12-01"},
            {"id": 1776, "__str__": "Aquaman #1", "cover_date": "1994-08-01"},
            {"id": 2429, "__str__": "Aquaman #1", "cover_date": "2003-02-01"},
            {"id": 2523, "__str__": "Aquaman #1", "cover_date": "2011-11-01"},
            {"id": 2896, "__str__": "Aquaman #1", "cover_date": "2016-08-01"},
        ],
    }
    return IssuesList(i)


def test_process_file(talker, fake_comic, issue_list_response, mocker):
    # Remove any existing metadata from comic fixture
    ca = ComicArchive(fake_comic)
    if ca.has_metadata():
        ca.remove_metadata()

    # Mock the call to Metron
    mocker.patch.object(Session, "issues_list", return_value=issue_list_response)
    talker._process_file(fake_comic, False)

    id, multiple = talker._process_file(fake_comic, False)
    assert id is None
    assert multiple
    assert fake_comic in [c.filename for c in talker.match_results.multiple_matches]

    id_list = []
    for c in talker.match_results.multiple_matches:
        for i in c.matches:
            id_list.append(i.id)

    assert 2471 in id_list


def test_write_issue_md(talker, fake_comic, metron_response, mocker):
    # Remove any existing metadata from comic fixture
    ca = ComicArchive(fake_comic)
    if ca.has_metadata():
        ca.remove_metadata()

    # Mock the call to Metron
    mocker.patch.object(Session, "issue", return_value=metron_response)
    talker.retrieve_single_issue(fake_comic, 5)

    # Now let's test writing the metadata to file
    talker._write_issue_md(fake_comic, 1)
    ca = ComicArchive(fake_comic)
    assert ca.has_metadata()
    ca_md = ca.read_metadata()
    assert ca_md.title == list_to_string([title for title in metron_response.story_titles])
    assert ca_md.series == metron_response.series.name
    assert ca_md.volume == str(metron_response.volume)
    assert ca_md.publisher == metron_response.publisher.name
    assert ca_md.issue == metron_response.number
    assert ca_md.story_arc == list_to_string([a.name for a in metron_response.arcs])
    assert ca_md.teams == list_to_string([t.name for t in metron_response.teams])
    assert ca_md.year == str(metron_response.cover_date.year)
    assert ca_md.characters == list_to_string([c.name for c in metron_response.characters])
    assert ca_md.credits is not None
    assert ca_md.credits[0]["person"] == "Al Milgrom"
    assert ca_md.credits[0]["role"] == "Cover"


def test_retrieve_single_issue(talker, fake_comic, metron_response, mocker):
    # Remove any existing metadata from comic fixture
    ca = ComicArchive(fake_comic)
    if ca.has_metadata():
        ca.remove_metadata()

    # Mock the call to Metron
    mocker.patch.object(Session, "issue", return_value=metron_response)
    talker.retrieve_single_issue(fake_comic, 10)

    # Now let's test the metadata
    ca = ComicArchive(fake_comic)
    assert ca.has_metadata()
    ca_md = ca.read_metadata()
    assert ca_md.title == list_to_string([title for title in metron_response.story_titles])
    assert ca_md.series == metron_response.series.name
    assert ca_md.volume == str(metron_response.volume)
    assert ca_md.publisher == metron_response.publisher.name
    assert ca_md.issue == metron_response.number
    assert ca_md.story_arc == list_to_string([a.name for a in metron_response.arcs])
    assert ca_md.teams == list_to_string([t.name for t in metron_response.teams])
    assert ca_md.year == str(metron_response.cover_date.year)
    assert ca_md.characters == list_to_string([c.name for c in metron_response.characters])
    assert ca_md.credits is not None
    assert ca_md.credits[0]["person"] == "Al Milgrom"
    assert ca_md.credits[0]["role"] == "Cover"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['metrontagger']

package_data = \
{'': ['*']}

install_requires = \
['darkseid>=1.0.6,<2.0.0', 'mokkari>=0.2.2,<0.3.0']

entry_points = \
{'console_scripts': ['metron-tagger = metrontagger.main:main']}

setup_kwargs = {
    'name': 'metron-tagger',
    'version': '1.1.6',
    'description': 'A program to write metadata from metron.cloud to a comic archive',
    'long_description': "=============\nMetron-Tagger\n=============\n\n.. image:: https://img.shields.io/pypi/v/metron-tagger.svg\n    :target: https://pypi.org/project/metron-tagger/\n\n.. image:: https://img.shields.io/pypi/pyversions/metron-tagger.svg\n    :target: https://pypi.org/project/metron-tagger/\n\n.. image:: https://img.shields.io/github/license/bpepple/metron-tagger\n    :target: https://opensource.org/licenses/GPL-3.0  \n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n\nQuick Description\n-----------------\n\nA command-line tool to tag comic archives with metadata from metron.cloud_.\n\n.. _metron.cloud: https://metron.cloud\n\nInstallation\n------------\n\nPyPi\n~~~~\n\nOr install it yourself:\n\n.. code:: bash\n\n  $ pip install --user metron-tagger\n\nGitHub\n~~~~~~\n\nInstalling the latest version from Github:\n\n.. code:: bash\n\n  $ git clone https://github.com/bpepple/metron-tagger.git\n  $ cd metron-tagger\n  $ python3 setup.py install\n\nFAQ\n---\n\n**Why no .cbr (rar) support?**\n\n- I'm not aware of any provider (Comixology, Humble Bundle, DriveThru Comics, etc.) of legally downloadable DRM-free comics that use the rar format.\n- It's a non-free software file format.\n- It is trivial to convert to cbz (zip) format.\n\nHelp\n----\n\n::\n\n    usage: metron-tagger [-h] [-r] [-o] [--id ID] [-d] [--ignore-existing]\n                         [--missing] [-u USER] [-p PASSWORD] [--set-metron-user]\n                         [-s] [--sort-dir SORT_DIR] [--set-sort-dir] [--version]\n                         path [path ...]\n\n    Read in a file or set of files, and return the result.\n\n    positional arguments:\n        path                  Path of a file or a folder of files.\n\n    optional arguments:\n        -h, --help            show this help message and exit\n        -r, --rename          Rename comic archive from the files metadata. (default: False)\n        -o, --online          Search online and attempt to identify comic archive. (default: False)\n        --id ID               Identify file for tagging with the Metron Issue Id. (default: None)\n        -d, --delete          Delete the metadata tags from the file. (default: False)\n        --ignore-existing     Ignore files that have existing metadata tag. (default: False)\n        --missing             List files without metadata. (default: False)\n        -u USER, --user USER  Metron user identity (default: None)\n        -p PASSWORD, --password PASSWORD\n                              Metron user password (default: None)\n        --set-metron-user     Save the Metron user settings (default: False)\n        -s, --sort            Sort files that contain metadata tags. (default: False)\n        --sort-dir SORT_DIR   Directory wher files should be sorted to. This should\n                              not have a trailing path serparator. (default: None)\n        --set-sort-dir        Set the directory where files should be sorted. (default: False)\n        --version             Show the version number and exit\n\nBugs/Requests\n-------------\n\nPlease use the `GitHub issue tracker <https://github.com/bpepple/metron-tagger/issues>`_ to submit bugs or request features.\n\nLicense\n-------\n\nThis project is licensed under the `GPLv3 License <LICENSE>`_.\n\n",
    'author': 'Brian Pepple',
    'author_email': 'bdpepple@gmail.com',
    'maintainer': 'Brian Pepple',
    'maintainer_email': 'bdpepple@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
